package com.ruoyi.app.mapper;

import java.util.List;

import com.ruoyi.app.DTO.TimeAndNumberDTO;
import com.ruoyi.app.DTO.ValidDistributeListDTO;
import com.ruoyi.app.DTO.ValidSlaughterCodeDTO;
import com.ruoyi.app.domain.DistributeBatch;
import com.ruoyi.app.domain.request.TimeRangeParam;

/**
 * 分销批次Mapper接口
 * 
 * @author coede
 * @date 2025-03-19
 */
public interface DistributeBatchMapper 
{
    /**
     * 查询分销批次
     * 
     * @param id 分销批次主键
     * @return 分销批次
     */
    public DistributeBatch selectDistributeBatchById(Long id);

    /**
     * 查询分销批次列表
     * 
     * @param distributeBatch 分销批次
     * @return 分销批次集合
     */
    public List<DistributeBatch> selectDistributeBatchList(DistributeBatch distributeBatch);

    /**
     * 校验分销批次的肉商是否唯一
     *
     * @param distributeBatch 分销批次
     * @return 结果
     */
    public DistributeBatch checkPurchaserUnique(DistributeBatch distributeBatch);

    /**
     * 获取指定入场批次的分销总量
     *
     * @param distributeBatch 分销批次
     * @return 结果
     */
    public Long getTotalByEntranceBatch(DistributeBatch distributeBatch);

    /**
     * 获取血码关联的未完成屠宰的分销批次
     *
     * @param slaughterCode 血码
     * @return 结果
     */
    public List<ValidDistributeListDTO> selectValidDistribute(String slaughterCode);

    /**
     * 新增分销批次
     * 
     * @param distributeBatch 分销批次
     * @return 结果
     */
    public int insertDistributeBatch(DistributeBatch distributeBatch);

    /**
     * 批量新增分销批次
     *
     * @param list 分销批次
     * @return 结果
     */
    public int insertDistributeBatchByBatch(List<DistributeBatch> list);

    /**
     * 修改分销批次
     * 
     * @param distributeBatch 分销批次
     * @return 结果
     */
    public int updateDistributeBatch(DistributeBatch distributeBatch);

    /**
     * 删除分销批次
     * 
     * @param id 分销批次主键
     * @return 结果
     */
    public int deleteDistributeBatchById(Long id);

    /**
     * 批量删除分销批次
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteDistributeBatchByIds(Long[] ids);

    /**
     * 根据时间获取分销次数
     *
     * @param timeRangeParam 时间
     * @return 结果
     */
    public Long selectCountByTime(TimeRangeParam timeRangeParam);

    /**
     * 根据时间获取累计屠宰
     *
     * @param timeRangeParam 时间
     * @return 结果
     */
    public Long selectAmountByTime(TimeRangeParam timeRangeParam);

    /**
     * 根据供应商ID统计分销批次数量
     *
     * @param supplierId 供应商ID
     * @return 结果
     */
    public int countBatchBySupplier(Long supplierId);

    /**
     * 根据肉商ID统计分销批次数量
     *
     * @param purchaserId 肉商ID
     * @return 结果
     */
    public int countBatchByPurchaser(Long purchaserId);

    /**
     * 根据血码统计分销批次数量
     *
     * @param slaughterCode 血码
     * @return 结果
     */
    public int countBatchBySlaughterCode(String slaughterCode);

    /**
     * 根据入场批次ID统计分销批次数量
     *
     * @param entranceBatchId 入场批次ID
     * @return 结果
     */
    public int countBatchByEntrance(Long entranceBatchId);


    /**
     * 获取今日分销次数
     *
     * @return 结果
     */
    public Long selectTodayCount();

    /**
     * 获取昨日分销次数
     *
     * @return 结果
     */
    public Long selectYesterdayCount();

    /**
     * 根据获取指定时间范围内每日累计计划屠宰
     *
     * @return 结果
     */
    public List<TimeAndNumberDTO> selectWeekCountByTime();

    /**
     * 获取存在未完成（24小时内）的分销批次的血码
     *
     * @return 结果
     */
    public List<ValidSlaughterCodeDTO> selectValidCode();

}
